package de.fhdw.wtf.tooling.editor.proposals;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.CompletionProposal;
import org.eclipse.jface.text.contentassist.ICompletionProposal;

public abstract class ContextProposal {
	public final String[] PROPOSALS;
	
	protected ContextProposal(final String[] proposals) {
		this.PROPOSALS = proposals;
	}
	
	public ICompletionProposal[] getProposals(final ITextViewer viewer, final int offset) {
		final Integer offsetToken = this.skipToToken(viewer, offset);
		if (offsetToken == null) {
			return null;
		}
		return this.getProposalsTransitively(viewer, offsetToken, offset);
	}
	
	protected abstract Integer skipToToken(ITextViewer viewer, int offset);
	
	protected String getQualifier(final IDocument document, final int documentOffset) {
		final StringBuffer buffer = new StringBuffer();
		int pos = documentOffset;
		while (true) {
			try {
				final char c = document.getChar(--pos);
				if (!Character.isWhitespace(c)) {
					buffer.append(c);
				}
				if (Character.isWhitespace(c)) {
					return buffer.reverse().toString();
				}
			} catch (final BadLocationException e) {
				return "";
			}
		}
	}
	
	protected ICompletionProposal[] getProposalsTransitively(final ITextViewer viewer,
			final int offsetColon,
			final int offset) {
		final IDocument document = viewer.getDocument();
		int documentOffset = offsetColon;
		
		while (true) {
			char c;
			try {
				if (documentOffset == 0) {
					return null;
				}
				c = document.getChar(--documentOffset);
				if (this.isInvalid(c, documentOffset, document)) {
					return null;
				}
				if (this.isValid(c, documentOffset, document)) {
					return this.getExplicitProposals(this.getQualifier(document, offset), offset);
				}
			} catch (final BadLocationException e) {
				e.printStackTrace();
			}
		}
	}
	
	private ICompletionProposal[] getExplicitProposals(final String qualifier, final int offset) {
		final int qualifierLength = qualifier.length();
		final Collection<ICompletionProposal> props = new Vector<>();
		for (final String proposal : this.PROPOSALS) {
			if (proposal == null) {
				break;
			}
			if (proposal.startsWith(qualifier)) {
				props.add(new CompletionProposal(proposal, offset - qualifierLength, qualifierLength, proposal.length()));
			}
		}
		final ICompletionProposal[] proposals = new ICompletionProposal[props.size()];
		return props.toArray(proposals);
	}
	
	protected abstract boolean isValid(char c, int offset, IDocument document);
	
	protected abstract boolean isInvalid(char c, int offset, IDocument document);
}
